@@ -1,66 +1,48 @@
-from xdist.dsession import DSession
+from xdist.dsession import DSession, get_default_max_worker_restart
 from xdist.report import report_collection_diff
-from xdist.scheduler import (
-    EachScheduling,
-    LoadScheduling,
-)
+from xdist.scheduler import EachScheduling, LoadScheduling
+from typing import Optional
 
-import py
 import pytest
 import execnet
 
-XSpec = execnet.XSpec
-
-
-def run(item, node, excinfo=None):
-    runner = item.config.pluginmanager.getplugin("runner")
-    rep = runner.ItemTestReport(item=item, excinfo=excinfo, when="call")
-    rep.node = node
-    return rep
-
 
 class MockGateway:
-    _count = 0
-
-    def __init__(self):
+    def __init__(self) -> None:
+        self._count = 0
         self.id = str(self._count)
         self._count += 1
 
 
 class MockNode:
-    def __init__(self):
-        self.sent = []
+    def __init__(self) -> None:
+        self.sent = []  # type: ignore[var-annotated]
         self.gateway = MockGateway()
         self._shutdown = False
 
-    def send_runtest_some(self, indices):
+    def send_runtest_some(self, indices) -> None:
         self.sent.extend(indices)
 
-    def send_runtest_all(self):
+    def send_runtest_all(self) -> None:
         self.sent.append("ALL")
 
-    def shutdown(self):
+    def shutdown(self) -> None:
         self._shutdown = True
 
     @property
-    def shutting_down(self):
+    def shutting_down(self) -> bool:
         return self._shutdown
 
 
-def dumpqueue(queue):
-    while queue.qsize():
-        print(queue.get())
-
-
 class TestEachScheduling:
-    def test_schedule_load_simple(self, testdir):
+    def test_schedule_load_simple(self, pytester: pytest.Pytester) -> None:
         node1 = MockNode()
         node2 = MockNode()
-        config = testdir.parseconfig("--tx=2*popen")
+        config = pytester.parseconfig("--tx=2*popen")
         sched = EachScheduling(config)
         sched.add_node(node1)
         sched.add_node(node2)
-        collection = ["a.py::test_1", ]
+        collection = ["a.py::test_1"]
         assert not sched.collection_is_completed
         sched.add_node_collection(node1, collection)
         assert not sched.collection_is_completed
@@ -70,19 +52,19 @@ class TestEachScheduling:
         assert sched.node2collection[node2] == collection
         sched.schedule()
         assert sched.tests_finished
-        assert node1.sent == ['ALL']
-        assert node2.sent == ['ALL']
+        assert node1.sent == ["ALL"]
+        assert node2.sent == ["ALL"]
         sched.mark_test_complete(node1, 0)
         assert sched.tests_finished
         sched.mark_test_complete(node2, 0)
         assert sched.tests_finished
 
-    def test_schedule_remove_node(self, testdir):
+    def test_schedule_remove_node(self, pytester: pytest.Pytester) -> None:
         node1 = MockNode()
-        config = testdir.parseconfig("--tx=popen")
+        config = pytester.parseconfig("--tx=popen")
         sched = EachScheduling(config)
         sched.add_node(node1)
-        collection = ["a.py::test_1", ]
+        collection = ["a.py::test_1"]
         assert not sched.collection_is_completed
         sched.add_node_collection(node1, collection)
         assert sched.collection_is_completed
@@ -96,8 +78,8 @@ class TestEachScheduling:
 
 
 class TestLoadScheduling:
-    def test_schedule_load_simple(self, testdir):
-        config = testdir.parseconfig("--tx=2*popen")
+    def test_schedule_load_simple(self, pytester: pytest.Pytester) -> None:
+        config = pytester.parseconfig("--tx=2*popen")
         sched = LoadScheduling(config)
         sched.add_node(MockNode())
         sched.add_node(MockNode())
@@ -120,8 +102,8 @@ class TestLoadScheduling:
         sched.mark_test_complete(node1, node1.sent[0])
         assert sched.tests_finished
 
-    def test_schedule_batch_size(self, testdir):
-        config = testdir.parseconfig("--tx=2*popen")
+    def test_schedule_batch_size(self, pytester: pytest.Pytester) -> None:
+        config = pytester.parseconfig("--tx=2*popen")
         sched = LoadScheduling(config)
         sched.add_node(MockNode())
         sched.add_node(MockNode())
@@ -147,8 +129,8 @@ class TestLoadScheduling:
         assert node1.sent == [0, 2, 4, 5]
         assert not sched.pending
 
-    def test_schedule_fewer_tests_than_nodes(self, testdir):
-        config = testdir.parseconfig("--tx=2*popen")
+    def test_schedule_fewer_tests_than_nodes(self, pytester: pytest.Pytester) -> None:
+        config = pytester.parseconfig("--tx=2*popen")
         sched = LoadScheduling(config)
         sched.add_node(MockNode())
         sched.add_node(MockNode())
@@ -167,8 +149,10 @@ class TestLoadScheduling:
         assert sent3 == []
         assert not sched.pending
 
-    def test_schedule_fewer_than_two_tests_per_node(self, testdir):
-        config = testdir.parseconfig("--tx=2*popen")
+    def test_schedule_fewer_than_two_tests_per_node(
+        self, pytester: pytest.Pytester
+    ) -> None:
+        config = pytester.parseconfig("--tx=2*popen")
         sched = LoadScheduling(config)
         sched.add_node(MockNode())
         sched.add_node(MockNode())
@@ -187,9 +171,9 @@ class TestLoadScheduling:
         assert sent3 == [2]
         assert not sched.pending
 
-    def test_add_remove_node(self, testdir):
+    def test_add_remove_node(self, pytester: pytest.Pytester) -> None:
         node = MockNode()
-        config = testdir.parseconfig("--tx=popen")
+        config = pytester.parseconfig("--tx=popen")
         sched = LoadScheduling(config)
         sched.add_node(node)
         collection = ["test_file.py::test_func"]
@@ -200,13 +184,13 @@ class TestLoadScheduling:
         crashitem = sched.remove_node(node)
         assert crashitem == collection[0]
 
-    def test_different_tests_collected(self, testdir):
+    def test_different_tests_collected(self, pytester: pytest.Pytester) -> None:
         """
         Test that LoadScheduling is reporting collection errors when
         different test ids are collected by workers.
         """
 
-        class CollectHook(object):
+        class CollectHook:
             """
             Dummy hook that stores collection reports.
             """
@@ -218,7 +202,7 @@ class TestLoadScheduling:
                 self.reports.append(report)
 
         collect_hook = CollectHook()
-        config = testdir.parseconfig("--tx=2*popen")
+        config = pytester.parseconfig("--tx=2*popen")
         config.pluginmanager.register(collect_hook, "collect_hook")
         node1 = MockNode()
         node2 = MockNode()
@@ -230,14 +214,15 @@ class TestLoadScheduling:
         sched.schedule()
         assert len(collect_hook.reports) == 1
         rep = collect_hook.reports[0]
-        assert 'Different tests were collected between' in rep.longrepr
+        assert "Different tests were collected between" in rep.longrepr
 
 
 class TestDistReporter:
-    @py.test.mark.xfail
-    def test_rsync_printing(self, testdir, linecomp):
-        config = testdir.parseconfig()
+    @pytest.mark.xfail
+    def test_rsync_printing(self, pytester: pytest.Pytester, linecomp) -> None:
+        config = pytester.parseconfig()
         from _pytest.pytest_terminal import TerminalReporter
+
         rep = TerminalReporter(config, file=linecomp.stringio)
         config.pluginmanager.register(rep, "terminalreporter")
         dsession = DSession(config)
@@ -249,6 +234,7 @@ class TestDistReporter:
         class gw2:
             id = "X2"
             spec = execnet.XSpec("popen")
+
         # class rinfo:
         #    version_info = (2, 5, 1, 'final', 0)
         #    executable = "hello"
@@ -259,48 +245,69 @@ class TestDistReporter:
         # linecomp.assert_contains_lines([
         #     "*X1*popen*xyz*2.5*"
         # ])
-        dsession.pytest_xdist_rsyncstart(source="hello", gateways=[gw1, gw2])
-        linecomp.assert_contains_lines(["[X1,X2] rsyncing: hello", ])
+        dsession.pytest_xdist_rsyncstart(source="hello", gateways=[gw1, gw2])  # type: ignore[attr-defined]
+        linecomp.assert_contains_lines(["[X1,X2] rsyncing: hello"])
 
 
-def test_report_collection_diff_equal():
+def test_report_collection_diff_equal() -> None:
     """Test reporting of equal collections."""
-    from_collection = to_collection = ['aaa', 'bbb', 'ccc']
+    from_collection = to_collection = ["aaa", "bbb", "ccc"]
     assert report_collection_diff(from_collection, to_collection, 1, 2) is None
 
 
-def test_report_collection_diff_different():
-    """Test reporting of different collections."""
-    from_collection = ['aaa', 'bbb', 'ccc', 'YYY']
-    to_collection = ['aZa', 'bbb', 'XXX', 'ccc']
-    error_message = (
-        'Different tests were collected between 1 and 2. The difference is:\n'
-        '--- 1\n'
-        '\n'
-        '+++ 2\n'
-        '\n'
-        '@@ -1,4 +1,4 @@\n'
-        '\n'
-        '-aaa\n'
-        '+aZa\n'
-        ' bbb\n'
-        '+XXX\n'
-        ' ccc\n'
-        '-YYY')
+def test_default_max_worker_restart() -> None:
+    class config:
+        class option:
+            maxworkerrestart: Optional[str] = None
+            numprocesses: int = 0
 
-    msg = report_collection_diff(from_collection, to_collection, '1', '2')
+    assert get_default_max_worker_restart(config) is None
+
+    config.option.numprocesses = 2
+    assert get_default_max_worker_restart(config) == 8
+
+    config.option.maxworkerrestart = "1"
+    assert get_default_max_worker_restart(config) == 1
+
+    config.option.maxworkerrestart = "0"
+    assert get_default_max_worker_restart(config) == 0
+
+
+def test_report_collection_diff_different() -> None:
+    """Test reporting of different collections."""
+    from_collection = ["aaa", "bbb", "ccc", "YYY"]
+    to_collection = ["aZa", "bbb", "XXX", "ccc"]
+    error_message = (
+        "Different tests were collected between 1 and 2. The difference is:\n"
+        "--- 1\n"
+        "\n"
+        "+++ 2\n"
+        "\n"
+        "@@ -1,4 +1,4 @@\n"
+        "\n"
+        "-aaa\n"
+        "+aZa\n"
+        " bbb\n"
+        "+XXX\n"
+        " ccc\n"
+        "-YYY"
+    )
+
+    msg = report_collection_diff(from_collection, to_collection, "1", "2")
     assert msg == error_message
 
 
 @pytest.mark.xfail(reason="duplicate test ids not supported yet")
-def test_pytest_issue419(testdir):
-    testdir.makepyfile("""
+def test_pytest_issue419(pytester: pytest.Pytester) -> None:
+    pytester.makepyfile(
+        """
         import pytest
 
         @pytest.mark.parametrize('birth_year', [1988, 1988, ])
         def test_2011_table(birth_year):
             pass
-    """)
-    reprec = testdir.inline_run("-n1")
+    """
+    )
+    reprec = pytester.inline_run("-n1")
     reprec.assertoutcome(passed=2)
     assert 0
